// Overcut for solid difference(), to assure complete holes.
overcut = 0.1;
exploded = 5.0;

include <Round-Anything/polyround.scad>

base_x = 20;
base_y = 74;
base_z = 4;
base_holes_distance = 52;

holder_x = 20;
holder_y = 130;
holder_z = 3;
holder_hole_distance = 117;

spacer_front_height = 20;
spacer_rear_height = 14;

side1_cut_angle = 78.5;
side2_cut_angle = 54.0;

join_rotation_angle = 24.2;

m8_hole_diam = 8.2;

baseRadiiPoints = [
    [0,           0, 6],
    [base_x,      0, 6],
    [base_x, base_y, 6],
    [0,      base_y, 6]
];

holderSide1Points = [
    [0,               0, 0,],
    [holder_x,        0, 0,],
    [holder_x, holder_y, 0,],
    [0,        holder_y, 5,],
];

holderSide2Points = [
    [0,                    0, 0,],
    [holder_x,             0, 0,],
    [holder_x,   holder_y-40, 9,],
    [holder_x/2, holder_y-32, 9,],
    [holder_x/2,    holder_y, 5,],
    [0,             holder_y, 0,],
];


// 2D shape of the holder base, with bolts holes.
module base_2d() {
    difference() {
        polygon(polyRound(baseRadiiPoints, 30));
        translate([base_x/2, (base_y-base_holes_distance)/2, 0]) circle(d=m8_hole_diam, $fn=64);
        translate([base_x/2, base_y-(base_y-base_holes_distance)/2, 0]) circle(d=m8_hole_diam, $fn=64);
    }
}

// 2D shape of the holder base, with bolt holes and angle marker.
module base_2d_cut() {
    difference() {
        base_2d();
        translate([base_x, base_y/2-8, 0])
            rotate([0, 0, join_rotation_angle]) {
                square([40, 0.2], center=true);
                square([0.2, 40], center=true);
            }
    }
}

// 3D holder base with bolt holes and welding angle marker.
module base() {
    difference() {
        translate([-base_x/2, -base_y/2, -base_z/2]) linear_extrude(height=base_z, convexity=10) base_2d();
        // The welding angle marker is made with two thin cuts.
        translate([holder_x/2, -8, 0])
            rotate([0, 0, join_rotation_angle]) {
                translate([-20, 0, base_z/2]) rotate([0, 90, 0]) cylinder(h=40, r=0.2);
                translate([0, 20, base_z/2])  rotate([90, 0, 0]) cylinder(h=40, r=0.2);
            }
    }
}


module side1_2d() {
    difference() {
        polygon(polyRound(holderSide1Points, 30));
        translate([holder_x/2, holder_hole_distance, 0]) circle(d=m8_hole_diam, $fn=64);
    }
}

module side2_2d() {
    polygon(polyRound(holderSide2Points, 30));
 }

module side1_2d_cut() {
    translate([holder_x, 0, 0])
        difference() {
            translate([-holder_x, 0, 0]) side1_2d();
            rotate([0, 0, side1_cut_angle+90]) square([holder_x*1.4, holder_x*1.4]);
        }
}

module side2_2d_cut() {
    difference() {
        side2_2d();
        rotate([0, 0, -side2_cut_angle]) square([holder_x*1.4, holder_x*1.4]);
    }
}

// Holder support arm.
module support_arm() {
    difference() {
        union() {
            translate([-holder_x, 0, -holder_z])            linear_extrude(holder_z, convexity=10) side1_2d();
            translate([-holder_z, 0, 0]) rotate([0, 90, 0]) linear_extrude(holder_z, convexity=10) side2_2d();
        }
        // Cube to make the diagonal cut.
        rotate([90-side2_cut_angle, 0, side1_cut_angle-90]) translate([-30, -20, -25]) cube([32, 20, 35]);
    }
}

// Cylindrical spacer.
module spacer(height) {
    color ("silver") linear_extrude(height) {
        difference() {
            circle(d=16, $fn=64);
            circle(d=m8_hole_diam, $fn=64);
        }
    }
}




// 3D layout.
color("silver" )base();
translate([holder_x/2, -8, base_z/2+exploded])
    rotate([0, 0, 21.2])
        rotate([side2_cut_angle, 0, 0])
            rotate([0, 0, 90-side1_cut_angle])
                color("silver") support_arm();

// Front spacer.
translate([0, base_holes_distance/2, -base_z/2-spacer_front_height-exploded]){
    spacer(spacer_front_height+overcut);
}

// Rear spacer.
translate([0, -base_holes_distance/2, -base_z/2-spacer_rear_height-exploded]) {
    spacer(spacer_rear_height+overcut);
}




/*
// 2D layout.
base_2d_cut();
translate([30, 0, 0]) side1_2d_cut();
translate([60, 0, 0]) side2_2d_cut();
*/